/* commandbytes */

#define	HDR 0x06  /* Variable-header (used to initiate a variabletransmission) */
#define RDY 0x09  /* Ready for data part                                       */
#define DAT 0x15  /* Data part (the datapart of a variable)                    */
#define	ABT 0x36  /* Answer skip/abort                                         */
#define	ACK 0x56  /* Package received OK (used to acknowledge a package)       */
#define	EOT 0x92  /* No more variables to send (used to end a transmission)    */
#define	SCR 0x6D  /* Send screendump                                           */

/* MESSAGE PROTOCOL

    hdr: xy 06 le nw le nd ty name..  lenw= number of bytes following
    dat: xy 15 le nw data..           lenv= lenw of following data block
    ack: xy 56 00 00                  85 name = len(byte) + chars
    rdy: xy 09 00 00                  82 name = chars + 00.. (8 bytes exactly)
    eot: xy 92 00 00
    scr: xy 6d 00 00

    pc: x = 0
    calc: x=8
    ti-82 y=2, ti-85 y=5



    DATABLOCK(S)                BACKUP          SCRREEN-DUMP
                Ŀ
    hdr                        hdr             scr
            ack                        ack             ack
            rdy                        rdy             data
    ack           1..n times  ack             ack
    data                       data
            ack                        ack
                              data
    eot                                 ack
                                data
                                        ack

FILE FORMATS

    **TI8x**etyCOMME            **TI8x**ety Back       **TI8x**ety Scre
    NT 41 CHARS ENDS            up=1 hdr + 3 dat       endump is data-p
    AT 0X34, 35 IS F            ablocks, 35 IS F       art only 35 IS F
    ILLENflhl.......            ILLENflhl.......       ILLENfldl.......
    ..dl............            ..dl............       ................
    ...                         ...                    ...
    .....hl.........            .....dl.........       ................
    .dl.............            .dl.............       .........CS
    ........CS                  ........CS          Screen dump has 'own'
                                                    file structure
    x = 2 | 5,  ty = 0A 00 | 0C 00 (for 82 resp 85)
    e = esc = 1B
    fl = filelen, hl = headerlen, dl = datalen CS = file checksum, (sum of bytes)

*/

/* The "Type"-byte (for a TI85) can have one of the following values (hex) */

#define	REAL 0x00	/* Real          */
#define	CPLX 0x01	/* Cplx          */
#define	VECT 0x02   /* Vectr         */
#define	VECX 0x03	/* Vectr complex */
#define	LIST 0x04   /* List          */
#define	LSTX 0x05	/* List  complex */
#define	MATR 0x06   /* Matrx         */
#define	MATX 0x07 	/* Matrx complex */
#define	CNST 0x08   /* Const         */
#define	CNSX 0x09 	/* Const complex */
#define	EQUN 0x0A   /* Equ           */
#define	RNGB 0x0B   /* Range         */
#define	STRG 0x0C   /* Strng         */
#define	GDBD 0x0D	/* GDB           */
#define	GDBE 0x0E	/* GDB           */
#define	GDBF 0x0F	/* GDB           */
#define	GDB0 0x10	/* GDB           */
#define	PICT 0x11   /* Pict          */
#define	PROG 0x12   /* Prgm          */
#define	RNG3 0x13   /* Range         */
                    /*               */
#define	RNG5 0x15   /* Range         */
#define	RNG6 0x16   /* Range         */
#define	RNG7 0x17   /* Range         */
#define	RNG8 0x18   /* Range         */
#define	RNG9 0x19   /* Range         */
#define	RNGa 0x1A   /* Range         */
#define	RNGb 0x1B   /* Range         */
#define	DUMP 0x1D   /* Memory dump   */

/* The "Type"-byte (for a TI82) can have one of the following values (hex) */

#define REAL2   0x00    /* Real          */ 
#define LIST2   0x01    /* List          */
#define MATR2   0x02    /* Matrix        */
#define EQUN2   0x03    /* Equation      */
                        /*               */
#define PROG2   0x05    /* Program       */
#define PROG3   0x06    /* Program?      */
#define PICT2   0x07    /* PIC?          */
                        /*               */
                        /*               */
                        /*               */
#define WIND2   0x0B    /* Window        */ 
#define ZSTO2   0x0C    /* Zsto          */
#define TBLE2   0x0D    /* Table         */
                        /*               */
#define DUMP2   0x0F    /* Memory dump   */ 

