/***************************************************************************/
/*  ThriftyLink (formerly TILINK) - originally written by Timo Stenberg,   */
/*  with many modifications & enhancements by Tobin Fricke (Light Ray)     */
/*  and a few improvements by Ashkan Ghaffari.                             */
/*  This software works with the "Four Buck Serial Link."                  */
/*                                                                         */
/*  This file touched by Light Ray - 04/08/96                              */
/*  And again by Ashkan Ghaffari   - 08/10/96                              */
/***************************************************************************/

/* Email: fricke@exo.com */

/*

  original header info:

    TI-link by Timo Stenberg

ver 1.0     released 22.11.1995

    fixed problem with 82-backups: handling of type vs type+0x20 taken care of
    (type+0x20 used only in header-dump)

	Changed 'copyright' into 'by'

ver 1.01	released 26.11.1996
     [Note: the above date is incorrect.  Probably should be 1995. - ttf]

ver 1.05
     touched up by Ashkan Ghaffari
     corrected problem with computer receiving variables
     corrected problem of duplicate file names (i.e. allows omitting of
	programs, renaming, overwriting, or aborting transmition)
     added option of transmitting multiple files at one time


*/

#include <setjmp.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <dir.h>
#include "tidef.h"
#include "output.c"               /* these are needed, but I put them in */
#include "tobin.c"                /* my project file.                    */
#include "tobin.h"
#include "output.h"

int dump_hdr(char * hdr);

typedef unsigned char uchar;
typedef unsigned int uint;

#define PEEK(b,o) (*(_f_p_ = MK_FP(b, o)))
unsigned far *_f_p_;

const char * types[] = {
"Real",	   /* 0 */
"Complex",
"Vector",
"Vector - Complex",
"List",
"List - Complex",
"Matrix",
"Matrix Complex",
"Const",
"Const Complex",
"Equation",
"Range",
"String",
"GDB",
"GDB",
"GDB",
"GDB",
"Pict",
"Program",
"Range",
"???",
"Range",
"Range",
"Range",
"Range",
"Range",
"Range",
"Range",
"???",
"Memory dump",  /* 1D */
"???",
"???",
"Real",	/* 20 start of 82 codes */
"List",
"Matrix",
"Equation",
"???",
"Program",
"Program",
"PIC?",
"???",
"???",
"???",
"Window", /* 20 + 0B */
"Zsto",
"Table",
"???",
"Memory dump", /* 2F */
"?",
"DATA-part"}; /* 31 */

#define DATATYPE 49

 jmp_buf jumb;
 int zcnt = 10000;
 uchar h[0x8100u];
 uint hl, last;
 uchar ctype  = 2;                  /* default calculator type is TI-82 */
 typedef struct {
	uchar	calc;
	uchar	cmnd;
	uint	hlen;
	uint	dlen;
	uchar	type;
	uchar	name[9];
 } header;
 FILE *file;
 uint port = 0x3f8;                 /* default serial port address    */

 char *def_sl1;
 char *def_sl2;

void updatestatusline(int background, char *sl1, char *sl2)
 { int x,y,a;
   x=wherex();
   y=wherey();
   window(1,24,80,25);
   textbackground(background);
   clrscr();
   gotoxy(1,1);
   output(sl1);
   gotoxy(1,2);
   output(sl2);
   window(1,2,80,23);
   gotoxy(x,y);
//   textattr=a;          /* Hmph.  Textattr was a variable in pascal. )-: */
 }

uchar bitavail(void) {                       /* written by light ray */
  return inport(port+6) & 2;
  /* looking at the schematic of the link and the source code here, it   */
  /* looks like DSR is used to receive data.  Bit 1 of MSR (modem status */
  /* register, [port+6]) is DDSR (delta DSR) and is set to one if DSR has*/
  /* changed since the last time DDSR has been read.  Reading MSR sets   */
  /* DDSR back to zero until DSR changes again...  Anyhow, there are a   */
  /* few flaws in this (ie consecutive similar bits) but it works for the*/
  /* most part...                                                        */
 }

/***********************************************************************
*
*   return next bit, check for break (input = 00 or shift key )
*
***********************************************************************/
uchar getbit(void) {

     uchar    b;

     if ((PEEK(0,0x417) % 4) == 3) { /* left and right shift key down */
		  cprintf("\n User break. \r\n");
		  longjmp(jumb, 1);
	 }
	 for (;;) {
	 b = inportb(port + 6) / 16 & 3;
		 if (!b) {
			  if (--zcnt == 0) {
					cprintf("\n break. \r\n");
					longjmp(jumb, 2);
			  }
		 }
		 else zcnt = 10000;  // funky timeout system ?? /light ray/
	 if (b == (inportb(port + 6) / 16 & 3)) break; /* slow down a fast PC */
	}
	return b;
}

/***********************************************************************
*
*   send a byte to calc
*
***********************************************************************/
void sendbyt(uchar b) {

    /* (port+4) is Modem Control Register (MCR).  Bit 0 is RTS and  */
    /* bit 1 is DTR.  DTR is tied to TI-white and RTS is tied to    */
    /* TI-red in the link.  - light ray - 			    */

    uchar bits;

    for (bits = 1; bits <= 8; bits++) {   /* eight bits == one byte   lr */
	if (b & 1) outportb(port + 4,2);  /* bit 1 of MCR is DTR      lr */
	else       outportb(port + 4,1);  /* bit 0 of MCR is RTS      lr */
	while (getbit() != 0);   /* wait for acknowledge from calc    ts */
	outportb(port + 4,3);    /* set DTR and RTS high              lr */
	while (getbit() != 3);   /*                                      */
	b >>= 1;    /* shift b right one bit and store result in b    lr */
    }
}

/***********************************************************************
*
*   get a byte from calc
*
***********************************************************************/
uchar recvbyt(void) {

    uchar curbit = 1;
    uchar byt = 0, bits, x;

    for (bits = 1; bits <= 8; bits++) {
	while ((x = getbit()) == 3);
	if (x == 1) {
	    byt += curbit;
	    outportb(port + 4,1);
	    x = 2;
	}
	else {
	    outportb(port + 4,2);
	    x = 1;
	}
	while ((getbit() & x) == 0);
	outportb(port + 4,3);
	curbit <<= 1;
    }
//    cprintf(".\r\n");
    return byt;
}

void talk(void)
 {int i;
  char c;
  int x1,x2,y1,y2;
  cprintf("Talk Started\r\n");
  for (i=0;i==14;i++) { cprintf("\r\n"); }
  updatestatusline(4," Talk "," Press both shift keys simultaneously to exit. ");
  window(1,17,80,23);
  textbackground(2);
  clrscr();
  window(1,10,80,16);
  textbackground(3);
  clrscr();
  textcolor(0);
  x1=1;
  y1=1;
  x2=1;
  y2=1;
  for (;;) {
   if (kbhit()) {
      window(1,10,80,16);
      textbackground(3);
      gotoxy(x1,y1);
      c=getche();
      sendbyt(c);
      if (c==0xD) cprintf("\n");
      x1=wherex();
      y1=wherey();
      }
   if (bitavail()){
      window(1,17,80,23);
      textbackground(2);
      gotoxy(x2,y2);
      putch(recvbyt());
      x2=wherex();
      y2=wherey();
     }
   if ((PEEK(0,0x417) % 4) == 3)
    {
      updatestatusline(4,def_sl1,def_sl2);
      textcolor(15);
      textbackground(1);
      clrscr();
      outportb(port + 4,3);                  /* make link active ???      */
      cprintf(" \r\n Ready. [Talk Terminated] \r\n");
      return;
    }
   }
 }

/***********************************************************************
*
*   send a short message to calc
*
***********************************************************************/
void sendack(uchar cmnd) {

    cprintf("<%2x%2x>", ctype, cmnd);
    sendbyt(ctype);
    sendbyt(cmnd);
    sendbyt(0);
    sendbyt(0);
}
/***********************************************************************
*
*   wait for an acknowledgement
*
***********************************************************************/
uchar getack(void) {

    uchar ack;

    (void)recvbyt();
    ack = recvbyt();
    (void)recvbyt();
    (void)recvbyt();
    return ack;
}

/***********************************************************************
*
*   receive a message into h[hl]
*
***********************************************************************/
int gethdr(void) {

    uint cks = 0, i, len;
    uint cl, ch;

    h[hl++] = recvbyt();
    h[hl++] = recvbyt();
    h[hl++] = recvbyt();
    h[hl++] = recvbyt();
    len = (uint)h[hl-1] * 256 + h[hl-2];
    if (h[hl-3] == ACK || h[hl-3] == EOT || h[hl-3] == RDY) len = 0;

    if (len > 0x8000) {
	cprintf("hdrlen too big: %d\r\n", len);
	return 0;
    }
    for (i = len; i; i--) {
	h[hl] = recvbyt();
	cks += h[hl++];
    }
    if (len) {
	cl = recvbyt();
	ch = recvbyt();
	cl += 256u*ch;
	if (cl != cks) {
	    cprintf("chksum error, calc=%d, recv=%d\r\n", cks, cl);
/*          return 0;*/
	}
    }
    cprintf(" Receiving header from calculator. ");
    return len;
}

/***********************************************************************
*
*   send a message from p, wait for an ack
*
***********************************************************************/
int sendhdr(header *p) {

    uint cks = 0, len, i;
    uchar *b;
    cprintf(" Sent header to calculator. \r\n");
//    cprintf("<%2x %2x %u>", p->calc, p->cmnd, p->hlen);
    sendbyt(p->calc);
    sendbyt(p->cmnd);
	len = p->hlen;
    sendbyt(len & 0xFF);
    sendbyt(len >> 8);
    b = (uchar *)p + 4;
    for (i = 0; i < len; i++) {
	sendbyt(b[i]);
	cks += (uint)b[i];
	}
    if (len) {
	sendbyt(cks);
	sendbyt(cks >> 8);
	}
    if (getack() != ACK) {
	cprintf("<err: ack=%2x>", cks);
	return 1;
    }
    return 0;   /* 0 = OK */
}
/***********************************************************************
*
*   receive data into h[0], if state = 5 receive screen dump,
*   else state should be 0 for data. Receives everything that is
*   sent from calc (eg. all programs)
*
***********************************************************************/
void recv (int state) {

    uint len, i;
    int j, type;

    hl = 0;     /* start receiving into h[0] */
    for (;;) {
	last = hl;
	len = gethdr();     /* hdr */
	cprintf("\r\n");
	if (len > 28) len = 28;   /* display max one row  */
//        for (i = 0; i < len + 4; i += 4) {
//            for (j = i; j < i + 4 && j < len + 4; j++) {
//                cprintf("%02x", h[last + j]);
//            }
//            if ((i & 0x1F) != 0x1C) cprintf(" ");
//            else cprintf("\r\n");                 /* 32 bytes per line */
//        }
	ctype = h[last + 0] & 15;	          /* calc type 85/82 */
        if (h[last + 1] != ACK) {                 /* dont ack ack */
	    sendack(ACK);
	    if (h[last + 1] == HDR) {             /* header */
	       state = 1;
	      sendack(RDY);   		          /* || is "or" */
	      type = dump_hdr(&h[last]);
	      if ((type == DUMP && ctype == 5)    /* TI-85 mem dump detected */
	       || (type == DUMP2 && ctype == 2))  /* TI-82 mem dump detected */
		{  state = 3;                  /* start receiving memory dump */
				}
		}
            else if (h[last + 1] == DAT) {      /* data */
			if (state == 1) state = 0;              /* ok, wait for more */
				else if (state >= 5) return;	/* screen or memory dump done */
				else if (state >1) state++;     /* memory dump in progress */
	        }
	    else if (h[last + 1] == EOT) {      /* no data */
		    return;
		}
	    }
		else {
			cprintf(" Data received. <ACK> \r\n");
	}
    } 							/* for */
} 							/* recv */

/***********************************************************************
*
*   dump header info
*
***********************************************************************/
int dump_hdr(char *hdr) {

    int t, c, l;
	char name[10];
    header *hp = (void *)hdr;

    if ((hp->calc & 0x7F) == 0x05) {
	c = '5';
		t = hp->type;
        l = hp->name[0];
	if (l > 9) l = 9;
	memcpy(name, &(hp->name[1]), l);
	name[l] = 0;
        l = hp->dlen;
    }
	else {
		c = '2';
		t = hp->type + 0x20;
		memcpy(name, hp->name, 8);
		name[8] = 0;
	l = hp->dlen;
    }
    if (hp->cmnd != HDR) {
	t = DATATYPE;
        strcpy(name, "--");
	l = hp->hlen;
    }
    cprintf("\r\n TI-8%c header: type %02x (%s), len %d, name %s.\r\n", c, hp->type, types[t], l, name);
    return hp->type;
}
/***********************************************************************
*
*   quick and dirty screen dump viewer, needs 3* 25 rows on PC
*
***********************************************************************/
int disp_scr(char *hdr) {

    int w, c, i, j, y;
    uchar pixl;
    uchar cc[4] = " ";
    header *hp = (void *)hdr;

    /* h = [ack] + screen data (= [0x 56 00 00] 0x 15 00 0L ddddd ) */
    w = ((hp->calc & 0x7F) == 0x02) ? 12 : 16;
    if (hdr[2] == 0 && (hdr[3] == 3 || hdr[3] == 4)) j = 4; /* screen dump */
    else j = 6;     /* hopefully pict-file */
    cprintf("\r\n");
    for (y = 0; y < hp->hlen / w; y++) {
        for (c = 0; c < w; c++) {
            pixl = hdr[j++];
	    for (i = 0; i < 4; i++) {
		cprintf("%c", cc[pixl >> 6]);
		pixl <<= 2;
            }
	}
	if (y % 25 == 24) {
	    if (getch() == 0x1B) return 1;
	}
	cprintf("\r\n");
    }
    return 0;
}

/***********************************************************************
*
*   prompts for a file for saving something: res = 1 if file OK
*
***********************************************************************/
int getfile(void) {
	char *fname;
	int res = 0;
	fname[0]=128;
	cprintf("\nSave to file? (empty quits): ");
	textcolor(11);
	cgets(fname);
	textcolor(15);
	fname += 2;                      /* same as fname = fname + 2 */
	if (strlen(fname)) {
	    if ((file = fopen(fname, "rt")) != NULL) {
	    fclose(file);
	    cprintf("File %s exists, overwrite (Y/N)? ",fname);
	    if ((getch() & 0xdf) != 'Y') { output("|11Yes.|15 \r\n"); return res;}
	       else output("|11No. |15\r\n ");
	}
	    if ((file = fopen(fname, "wb")) == NULL) {
	    cprintf(" Can't open file %s.\r\n", fname);
	}
	else {
	    res = 1;
	}
	}
	cprintf("\r\n");
	return res;
}

/***********************************************************************
*
*   mode: 0 = all blocks, 1 = screen dump, 2 = current block
*
***********************************************************************/
void savefile(int mode, header *hp) {

    uint fsize, dump = 0;
    uint cks = 0, i;
    uchar *b;

    if (mode != 1) {
	fprintf(file, "**TI8%c**%c%c%c",ctype + '0', 0x1a, (ctype == 5) ? 12 : 10, 0);
		fprintf(file, "Comment 42 chars                         .");
	fwrite(&hp->hlen, 1, 2, file);  /* dummy filelen */
    }
    else {  /* proprietary file format for screen dump */
	fprintf(file, "**TI8%c**%c%c%c",ctype + '0', 0x1a, (ctype == 5) ? 12 : 10, 0);
		fprintf(file, "Screen dump                              .");
	i = (ctype == 5) ? 1026 : 770;
	fwrite(&i, 1, 2, file);  /* filelen */
    }
	fsize = 0;
	for (;;) {
	if ((hp->calc & 0x7F) != ctype) {
	    if (!dump) cprintf("Dataerror %2x at %p", hp->calc, hp);
	    break;  /* ok if backup */
		}
	if (hp->cmnd == EOT) break; /* normal exit */
        if (hp->cmnd == HDR || hp->cmnd == DAT) {
	    if (hp->cmnd == HDR &&
	      ((hp->type == DUMP && ctype == 5)
	    || (hp->type == DUMP2 && ctype == 2))) {
                dump = 1;
                mode = 0;   /* can't have only one block for dump */
	    }
	    fsize += fwrite(&hp->hlen, 1, hp->hlen + 2, file);
			if (mode == 1) {	/* screen */
				return;
			}
	    b = (void *)&hp->hlen;
	    for (i = 0;i < hp->hlen + 2 ;i++ ) {    /* calculate checksum */
		cks += b[i];
	    }
	    if (hp->cmnd == DAT && mode == 2) break;   /* only one block */
	}
	hp = (void *)((char *)hp + hp->hlen + 4);   /* advance pointer */
    }
    fwrite(&cks, 1, 2, file);   /* checksum */
	fseek(file, 0x35, SEEK_SET);
	fwrite(&fsize, 1, 2, file);	/* actual filelen at 0x35*/
}

void sendfile(void) {

    header *hp;
    uint fsize, blks, sblks = 0;
    int dump = 0;
    char *fname;
    hp = (void *)h;
    do {
    cprintf("\n Send what file? (empty begins transmission): ");
    textcolor(11);
    fname[0]=128; /* max length of filename for cgets */
    cgets(fname);
    textcolor(15);
    fname += 2;
    if (strlen(fname)) {
       if ((file = fopen(fname, "rb")) == NULL) {
	cprintf("File %s does not exist.\r\n",fname);
	}
	else { cprintf("\r\n");
    fread(hp, 1, 0x35, file);    /* comments and calc type */
	ctype = (hp+9 == 12) ? 5 : 2;
	fread(&fsize, 1, 2, file);
	blks = 0; /* header */
    while(fsize > 12) {
	hp->calc = ctype;
		fread(&hp->hlen, 1, 2, file);
	if (hp->hlen + 2 > fsize) {cprintf("err:fsize %d", fsize); return;}
	fsize -= (hp->hlen + 2);
		if (fread(&hp->dlen, 1, hp->hlen, file) < hp->hlen) {cprintf("err:nodata"); return;}
	if (blks == 0) {    /* normally header+data pairs */
			hp->cmnd = HDR;
	    if ((hp->type == DUMP && ctype == 5)
	    || (hp->type == DUMP2 && ctype == 2)) {
		blks = 3;
		dump = 1;
	    }
	    else blks = 1;
	}
	else {
	    blks--;
	    hp->cmnd = DAT;
	}
	sblks++;
	hp = (void *)((char *)hp + hp->hlen + 4);
    }
    fclose(file);
    }
    }
    } while (strlen(fname));
    hp->calc = ctype;           /* needed if re-saved */
    hp->cmnd = EOT;
    hl = (uchar *)hp - (uchar *)h;

    /* now file has been read into h, send it */

    hp = (void *)h;
    while(sblks) {
	hp->calc = ctype;
	fsize -= (hp->hlen + 2);
	if (sendhdr(hp)) {cprintf("Error: sendblk"); return;}
	if (hp->cmnd == HDR) {
	    if (getack() == ABT) {
	      uchar temp2 = recvbyt();
		if (temp2==03)
		  {cprintf("Error: Calculator out of memory\r\n"); return;}
		recvbyt();
		recvbyt();
		if (temp2==01)
		  {cprintf("Error: Calculator Abort"); return;}
		  else {
		   cprintf("Variable omitted");
		   sblks--;
		   hp = (void *)((char *)hp + hp->hlen + 4);
		  }
	    }
		sendack(ACK);
	}
	sblks--;
	hp = (void *)((char *)hp + hp->hlen + 4);
    }
    if (!dump) {
        sendack(EOT);
        (void)getack();
    }
    cprintf(" OK\r\n Data transfered successfully.\r\n");
}


void init(void)
 { def_sl1="\n|12[|14L|12]|15ist files |12[|14C|12]|15hange Directory  |12[|14B|12]|15atch Receive\r";
   def_sl2="\n|12[|14Q|12]|15uit, |12[|14S|12]|15creen, |12[|14D|12]|15isp.buf, |12[|14V|12]|15iew.buf, |12[|14R|12]|15ecv, |12[|14X|12]|15mit, TI-8|12[|145|12/|142|12]|15, |12[|14H|12]|15elp";
  }


/***********************************************************************
*
*   Help screen
*
***********************************************************************/
void syntax(char *name) {

  output("\r\nAuthors: Timo Stenberg  <timo.stenberg@ntc.nokia.com>"
	 "\r\n         Tobin Fricke   <fricke@exo.com>"
	 "\r\n         Ashkan Ghaffari <ghaff@erols.com>\r\n\r\n");

  cprintf("--pause--\r");
  while (!kbhit());
  getch();

  output("Help:           \r\n"
	 " [Q]uit     Exit the program\r\n"
         " [S]creen   Request screen dump from TI8x, saved as file with no header\r\n"
	 " [D]isplay  Show a graphical view of current buffer contents\r\n"
	 " [V]iew.buf Get info of current buffer contents, save individual components\r\n"
	 "            of a nnnnn.8xG - file\r\n"
	 " [R]ecv     Receive from TI8x into the buffer and save to a file\r\n"
         " [X]mit     Send a file to TI8x, leave a copy in the buffer. Combination\r\n"
         "            X+V and X+D work even without a calculator connected\r\n"
         " [5]/[2]    Select calculator type\r\n"
         " [L]ist     List *.85* files in the current directory\r\n"
         " [C]hDir    Change directory\r\n"
         " [H]elp:    show this text\r\n\r\n");

    cprintf("--pause--\r");
    while (!kbhit());
    getch();

    cprintf("            \r\n"
     "Command line: %s [ANYTHING] \r\n"
     "  BRINGS UP COMPORT SELECT, OTHERWISE com1 DEFAULT\r\n"
     "        BE CAREFUL no checking performed\r\n", name);
 }

uint getport(void) {
 char c;
 int p;
 uint a;

 output("\r\n");
 output("|15 ThriftyLink!  Please Choose A Serial Port.  \r\n");
 output("|14 com1         com2         com3         com4  ");
 p=1;
 c=0;
 a=0;
 while (c!=0xD) {
   gotoxy(13*(p-1)+2,wherey());
   textcolor(15);
   textbackground(3);
   cprintf("com%u",p);
   while (!kbhit);
   c=getch();
   if (c==0) {
     while (!kbhit);
     c=getch();
     gotoxy(13*(p-1)+2,wherey());
     textcolor(14);
     textbackground(1);
     cprintf("com%u",p);
     switch (c) {
      case 77: { p++; if (p>4) p=1; } break;
      case 75: { p--; if (p<1) p=4; } break;
      case 31: p=1; break;
      case 32: p=2; break;
      case 33: p=3; break;
      case 34: p=4; break;
      } /* switch */
     } /* if c==0 */
    else
     { if (c==0x1B) exit(0);
       if (c==0xD) {
          if (p==1) a=0x3f8;
	  if (p==2) a=0x2f8;
	  if (p==3) a=0x3e8;
	  if (p==4) a=0x2e8;
	  break;
	  }
        }
     }
  textcolor(15);
  textbackground(1);
  cprintf("\r\n\n");
  return a;
  }

void getcmdline(int argc)
 {
    if (argc!=1) port=getport();
}

/***********************************************************************
*
*   MAIN; select port, wait and perform user commands
*
***********************************************************************/
int main(int argc, char* argv[]) {
    uint i, j;
    int chr;

    /* long jump moved to after the screen is made nice -TTF- */

    outportb(port + 4,3);                  /* make link active ???      */

    init();
    clrscr();                              /* initial clearscreen -TTF- */
    window(1,1,80,1);
    textbackground(4);
    textcolor(15);
    clrscr();
    cprintf("ThriftyLink (TILINK) v1.05 by Timo Stenberg, Tobin Fricke, and Ashkan Ghaffari");
    updatestatusline(4,def_sl1,def_sl2);
    textbackground(1);
    clrscr();

    output(" \n   |14ͻ\r\n");                   // TTF
    output(  "    |14|11  ThriftyLink (TILINK) V1.05    |14     \r\n");
    output(  "    |14                                     \r\n");
    output(  "    |14ͼ|15\r\n\r\n");

    getcmdline(argc);

    cprintf(" Serial-port address: 0x0%X\r\n", port);
    cprintf(" Hit both SHIFT keys to break out of an unsuccesful operation. \r\n");

    j = setjmp(jumb);                      /* longjmp returns here */
    cprintf("\n Ready.\n\r");

    for (;;) {
	chr = getch() | 0x20;   /* lcase */
    //    putchar(chr);

        if (chr == 'c') {
	  cprintf(" Change to what directory? >");
	   { char *dir;
	     dir[0]=128;
	     textcolor(11);
	     cgets(dir);
             textcolor(15);
             dir++;
             dir++;
             if (chdir(dir))
                 perror("chdir()");
	     else
	       { getcurdir(0,dir);
		 cprintf("\r Directory changed to %c:\\%s.\n\r       ",getdisk() + 'A',dir);
	       }
           }
          }

	else if (chr == 's') {
	    sendack(SCR);
            recv(5);
	    (void)disp_scr(h);
            if (getfile()) {
		savefile(1, (void *)h);
			fclose(file);
	        	cprintf("Done!\r\n");
        	}
        }
        else if (chr == 'd') {
            for (i = 0; i < hl;) {
                if (disp_scr(&h[i])) break;
                j = h[i+3];
		j = (j << 8) + h[i+2];
		i = i + j + 4;
	    }
        }
        else if (chr == 'q') {
	    window(1,1,80,25);
	    textcolor(15);
	    textbackground(0);
            clrscr();
	    cprintf(" Thank you for using ThriftyLink. \r\n\r\n");
            textcolor(7);
	    exit(1);
	}
        else if (chr == 'v') { /* dump all */
            cprintf("\n");
            if (hl < 30) hl = 30;
            for (i = 0; i < hl;) {
                (void)dump_hdr(&h[i]);
                if (h[i+1] == HDR && getfile()) {
                    savefile(2, (void *)&h[i]);
		    fclose(file);
		}
		j = h[i+3];
                j = (j << 8) + h[i+2];
                i = i + j + 4;
	    }
	}

        else if (chr == 'l')
	    listfiles("*.8??");

	else if (chr == 'r') {
	    cprintf("\nWaiting...");
            recv(0); /* What does this do?  Clear buffers? */
			if (getfile()) {
			 savefile(0, (void *)h);
		         fclose(file);
	        	 cprintf("Done!\r\n");
        	}
        }
	else if (chr == 'b') {
	    cprintf("\n Batch Super-Receive Mode - Press both SHIFT keys to Terminate.\n\r");
	    cprintf(" not implemented yet\n\r");
            /* This function should do a continuous receive, automatically
               saving files as xxxxxxxx.8yz, where xxxxxxx is the name of
	       whatever is transfered, y is ctype (2 or 5), and z is the
	       first letter of the type. (P=program, B=backup, etc) */
	}
	else if (chr == 'x') {
			sendfile();
		}
	else if (chr == '2') {
	    output(" Protocol set to TI-82. \r\n");
                   ctype = 2;     /* calctype = TI-82 */
        }
        else if (chr == '5') {
            output(" Protocol set to TI-85. \r\n");
	    ctype = 5;     /* calctype = TI-85 */
        }
        else if (chr == 'h') {
            syntax(argv[0]);
        }
	else if (chr == 't') { talk(); }
    }
}
